@@ -12,6 +12,11 @@
  * Author: Jeff Hill
  */
 
+/* pull in _WIN32_WINNT* definitions, if _WIN32_WINNT
+ * is not already defined it is set to max supported by SDK
+ */
+#include <sdkddkver.h>
+
 #include <string.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -32,14 +37,58 @@
 #include "epicsAtomic.h"
 #include "osdThreadPvt.h"
 
+/* Windows Vista and higher supports fibre functions, but the
+ * prototypes only appear in the windows SDK 8 and above.
+ * VS2010 supplies sdk 7, but can be upgraded to later SDK 
+
+ * To accomodate this we suuply prototypes on, for XP
+ * fall back to Tls*() which will build and run
+ * correctly for epicsThreads, but means that TLS allocations from
+ * epicsThreadImplicitCreate() will continue to leak (for non-EPICS threads).
+ *
+ * Also, WINE circa 5.0.3 provides the FLS storage functions, but doesn't
+ * actually run the dtor function.
+ *
+ * we check for existance of _WIN32_WINNT_WIN8 which will only be defined
+ * in SDK 8 and above. If Visa is detected and SDK < 8 we will supply
+ * the missing prototypes
+ */
+
+#if _WIN32_WINNT >= 0x0600 /* VISTA */
+#   ifdef _WIN32_WINNT_WIN8 /* Existance means using SDK 8 or higher */
+#       include <fibersapi.h>
+#   else
+#       include <winnt.h> /* for PFLS_CALLBACK_FUNCTION */
+/* the fibers api exists in vista and above, but no header is provided until SDK 8 */
+        WINBASEAPI DWORD WINAPI FlsAlloc(PFLS_CALLBACK_FUNCTION);
+        WINBASEAPI PVOID WINAPI FlsGetValue(DWORD);
+        WINBASEAPI BOOL WINAPI FlsSetValue(DWORD , PVOID);
+        WINBASEAPI BOOL WINAPI FlsFree(DWORD);
+#   endif
+#elif _WIN32_WINNT >= 0x0501 /* Windows XP */
+    typedef void (WINAPI *xPFLS_CALLBACK_FUNCTION) (void*);
+    static
+    DWORD xFlsAlloc(xPFLS_CALLBACK_FUNCTION dtor) {
+        (void)dtor;
+        return TlsAlloc();
+    }
+#   define FlsAlloc xFlsAlloc
+#   define FlsSetValue TlsSetValue
+#   define FlsGetValue TlsGetValue
+#   define USE_TLSALLOC_FALLBACK
+#else
+#   error Minimum supported is Windows XP
+#endif
+
 LIBCOM_API void osdThreadHooksRun(epicsThreadId id);
 
 void setThreadName ( DWORD dwThreadID, LPCSTR szThreadName );
+static void WINAPI epicsParmCleanupWIN32 ( void * praw );
 
 typedef struct win32ThreadGlobal {
     CRITICAL_SECTION mutex;
     ELLLIST threadList;
-    DWORD tlsIndexThreadLibraryEPICS;
+    DWORD flsIndexThreadLibraryEPICS;
 } win32ThreadGlobal;
 
 typedef struct epicsThreadOSD {
@@ -53,6 +102,7 @@ typedef struct epicsThreadOSD {
     unsigned epicsPriority;
     char isSuspended;
     int joinable;
+    int isRunning;
     HANDLE timer; /* waitable timer */
 } win32ThreadParam;
 
@@ -119,7 +169,6 @@ BOOL WINAPI DllMain (
          * Don't allow user's explicitly calling FreeLibrary for Com.dll to yank
          * the carpet out from under EPICS threads that are still using Com.dll
          */
-#if _WIN32_WINNT >= 0x0501
         /*
          * Only in WXP
          * That's a shame because this is probably much faster
@@ -127,22 +176,7 @@ BOOL WINAPI DllMain (
         success = GetModuleHandleEx (
             GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
             ( LPCTSTR ) DllMain, & dllHandle );
-#else
-        {
-            char name[256];
-            DWORD nChar = GetModuleFileName (
-                hModule, name, sizeof ( name ) );
-            if ( nChar && nChar < sizeof ( name ) ) {
-                dllHandle = LoadLibrary ( name );
-                if ( ! dllHandle ) {
-                    success = FALSE;
-                }
-            }
-            else {
-                success = FALSE;
-            }
-        }
-#endif
+
         if ( success ) {
             success = TlsSetValue ( dllHandleIndex, dllHandle );
         }
@@ -207,8 +241,8 @@ static win32ThreadGlobal * fetchWin32ThreadGlobal ( void )
 
     InitializeCriticalSection ( & pWin32ThreadGlobal->mutex );
     ellInit ( & pWin32ThreadGlobal->threadList );
-    pWin32ThreadGlobal->tlsIndexThreadLibraryEPICS = TlsAlloc();
-    if ( pWin32ThreadGlobal->tlsIndexThreadLibraryEPICS == 0xFFFFFFFF ) {
+    pWin32ThreadGlobal->flsIndexThreadLibraryEPICS = FlsAlloc(&epicsParmCleanupWIN32);
+    if ( pWin32ThreadGlobal->flsIndexThreadLibraryEPICS == FLS_OUT_OF_INDEXES ) {
         DeleteCriticalSection ( & pWin32ThreadGlobal->mutex );
         free ( pWin32ThreadGlobal );
         pWin32ThreadGlobal = 0;
@@ -233,10 +267,14 @@ static void epicsParmCleanupDataWIN32 ( win32ThreadParam * pParm )
     }
 }
 
-static void epicsParmCleanupWIN32 ( win32ThreadParam * pParm )
+static void WINAPI epicsParmCleanupWIN32 ( void * praw )
 {
-    win32ThreadGlobal * pGbl = fetchWin32ThreadGlobal ();
-    if ( ! pGbl )  {
+    win32ThreadParam * pParm = praw;
+    win32ThreadGlobal * pGbl;
+    if(!praw)
+        return;
+
+    if ( ! (pGbl = fetchWin32ThreadGlobal ()) )  {
         fprintf ( stderr, "epicsParmCleanupWIN32: unable to find ctx\n" );
         return;
     }
@@ -464,7 +502,7 @@ static unsigned WINAPI epicsWin32ThreadEntry ( LPVOID lpParameter )
     if ( pGbl )  {
         setThreadName ( pParm->id, pParm->pName );
 
-        success = TlsSetValue ( pGbl->tlsIndexThreadLibraryEPICS, pParm );
+        success = FlsSetValue ( pGbl->flsIndexThreadLibraryEPICS, pParm );
         if ( success ) {
             osdThreadHooksRun ( ( epicsThreadId ) pParm );
             /* printf ( "starting thread %d\n", pParm->id ); */
@@ -482,14 +520,16 @@ static unsigned WINAPI epicsWin32ThreadEntry ( LPVOID lpParameter )
 
     epicsExitCallAtThreadExits ();
 
-    /*
-     * CAUTION: !!!! the thread id might continue to be used after this thread exits !!!!
-     */
-    if ( pGbl )  {
-        TlsSetValue ( pGbl->tlsIndexThreadLibraryEPICS, (void*)0xdeadbeef );
-    }
+    epicsAtomicSetIntT(&pParm->isRunning, 0);
 
-    epicsParmCleanupWIN32 ( pParm );
+    /* On Windows we could omit this and rely on the callback given to FlsAlloc() to free.
+     * However < vista doesn't implement FLS at all, and WINE (circa 5.0.3) doesn't
+     * implement fully (dtor never runs).  So for EPICS threads, we explicitly
+     * free() here.
+     */
+    if(pGbl && FlsSetValue ( pGbl->flsIndexThreadLibraryEPICS, NULL )) {
+        epicsParmCleanupWIN32 ( pParm );
+    }
 
     return retStat; /* this indirectly closes the thread handle */
 }
@@ -503,6 +543,7 @@ static win32ThreadParam * epicsThreadParmCreate ( const char *pName )
         pParmWIN32->pName = (char *) ( pParmWIN32 + 1 );
         strcpy ( pParmWIN32->pName, pName );
         pParmWIN32->isSuspended = 0;
+        pParmWIN32->isRunning = 1;
         epicsAtomicIncrIntT(&pParmWIN32->refcnt);
 #ifdef CREATE_WAITABLE_TIMER_HIGH_RESOLUTION
         pParmWIN32->timer = CreateWaitableTimerEx(NULL, NULL, CREATE_WAITABLE_TIMER_HIGH_RESOLUTION, TIMER_ALL_ACCESS);
@@ -550,7 +591,7 @@ static win32ThreadParam * epicsThreadImplicitCreate ( void )
         win32ThreadPriority = GetThreadPriority ( pParm->handle );
         assert ( win32ThreadPriority != THREAD_PRIORITY_ERROR_RETURN );
         pParm->epicsPriority = epicsThreadGetOsiPriorityValue ( win32ThreadPriority );
-        success = TlsSetValue ( pGbl->tlsIndexThreadLibraryEPICS, pParm );
+        success = FlsSetValue ( pGbl->flsIndexThreadLibraryEPICS, pParm );
         if ( ! success ) {
             epicsParmCleanupWIN32 ( pParm );
             pParm = 0;
@@ -638,6 +679,8 @@ epicsThreadId epicsThreadCreateOpt (
         return NULL;
     }
 
+    /* after this point, new thread is responsible to free(pParmWIN32) */
+
     return ( epicsThreadId ) pParmWIN32;
 }
 
@@ -685,7 +728,7 @@ static void* getMyWin32ThreadParam ( win32ThreadGlobal * pGbl )
     }
 
     pParm = ( win32ThreadParam * )
-        TlsGetValue ( pGbl->tlsIndexThreadLibraryEPICS );
+        FlsGetValue ( pGbl->flsIndexThreadLibraryEPICS );
     if ( ! pParm ) {
         pParm = epicsThreadImplicitCreate ();
     }
@@ -1005,6 +1048,8 @@ static void epicsThreadShowInfo ( epicsThreadId id, unsigned level )
             fprintf (epicsGetStdout(), " %-8p %-8p ",
                 (void *) pParm->handle, (void *) pParm->parm );
         }
+        if(!epicsAtomicGetIntT(&pParm->isRunning))
+            fprintf (epicsGetStdout(), " ZOMBIE");
     }
     else {
         fprintf (epicsGetStdout(),
@@ -1046,6 +1091,11 @@ LIBCOM_API void epicsStdCall epicsThreadShowAll ( unsigned level )
     win32ThreadGlobal * pGbl = fetchWin32ThreadGlobal ();
     win32ThreadParam * pParm;
 
+#ifdef USE_TLSALLOC_FALLBACK
+    fprintf(epicsGetStdout(), "Warning: For this target, use of epicsThread* from non-EPICS threads\n"
+                              "         May leak memory.  Recommend to upgrade to >= Window Vista\n");
+#endif
+
     if ( ! pGbl ) {
         return;
     }
